"use strict";
"use client";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PdfHighlighter = void 0;
const react_1 = __importStar(require("react"));
const pdf_viewer_1 = require("pdfjs-dist/legacy/web/pdf_viewer");
require("pdfjs-dist/web/pdf_viewer.css");
require("../style/pdf_viewer.css");
require("../style/PdfHighlighter.css");
const lodash_debounce_1 = __importDefault(require("lodash.debounce"));
const client_1 = require("react-dom/client");
const coordinates_1 = require("../lib/coordinates");
const get_area_as_png_1 = __importDefault(require("../lib/get-area-as-png"));
const pdfjs_dom_1 = require("../lib/pdfjs-dom");
const TipContainer_1 = require("./TipContainer");
const PdfHighlighter = ({ highlightTransform, highlights, onScrollChange, scrollRef, pdfDocument, pdfScaleValue = "auto", }) => {
    const [tip, setTip] = (0, react_1.useState)(null);
    const [tipPosition, setTipPosition] = (0, react_1.useState)(null);
    const [tipChildren, setTipChildren] = (0, react_1.useState)(null);
    const [scrolledToHighlightId, setScrolledToHighlightId] = (0, react_1.useState)("");
    const [containerNode, setContainerNode] = (0, react_1.useState)(null);
    const documentRef = react_1.default.useRef(null);
    const [viewerState, setViewerState] = (0, react_1.useState)();
    const [eventBus] = (0, react_1.useState)(new pdf_viewer_1.EventBus());
    var linkService;
    var viewer;
    var roots = new Map();
    const handleScaleValue = () => {
        if (viewer !== null && viewer !== undefined) {
            viewer.currentScaleValue = pdfScaleValue;
        }
    };
    const debouncedScaleValue = (0, react_1.useCallback)((0, lodash_debounce_1.default)(handleScaleValue, 500), []);
    const resizeObserver = new ResizeObserver(debouncedScaleValue);
    (0, react_1.useEffect)(() => {
        var _a;
        const current = documentRef.current;
        setContainerNode(current);
        let doc;
        if (current) {
            let { ownerDocument: doc } = current;
            eventBus.on("textlayerrendered", () => {
                renderHighlights();
            });
            eventBus.on("pagesinit", () => {
                handleScaleValue();
                scrollRef(scrollTo);
            });
            (_a = doc === null || doc === void 0 ? void 0 : doc.defaultView) === null || _a === void 0 ? void 0 : _a.addEventListener("resize", debouncedScaleValue);
            if (resizeObserver)
                resizeObserver.observe(current);
            linkService = new pdf_viewer_1.PDFLinkService({
                eventBus: eventBus,
                externalLinkTarget: 2,
            });
            viewer = new pdf_viewer_1.PDFViewer({
                container: documentRef.current,
                eventBus: eventBus,
                textLayerMode: 2,
                removePageBorders: true,
                linkService: linkService,
                l10n: pdf_viewer_1.NullL10n,
            });
            setViewerState(viewer);
        }
        return () => {
            var _a;
            eventBus.off("textlayerrendered", renderHighlights);
            eventBus.off("pagesinit", () => {
                handleScaleValue();
                scrollRef(scrollTo);
            });
            (_a = doc === null || doc === void 0 ? void 0 : doc.defaultView) === null || _a === void 0 ? void 0 : _a.removeEventListener("resize", debouncedScaleValue);
        };
    }, []);
    (0, react_1.useEffect)(() => {
        if (viewer) {
            renderHighlights();
            viewer.setDocument(pdfDocument);
            linkService.setViewer(viewer);
            handleScaleValue();
        }
    }, [containerNode]);
    (0, react_1.useEffect)(() => {
        renderHighlights();
    }, [tip]);
    const groupHighlightsByPage = (highlights) => {
        const allHighlights = highlights.filter(Boolean);
        const pageNumbers = new Set();
        for (const highlight of allHighlights) {
            pageNumbers.add(highlight.position.pageNumber);
            for (const rect of highlight.position.rects) {
                if (rect.pageNumber) {
                    pageNumbers.add(rect.pageNumber);
                }
            }
        }
        const groupedHighlights = {};
        for (const pageNumber of pageNumbers) {
            groupedHighlights[pageNumber] = groupedHighlights[pageNumber] || [];
            for (const highlight of allHighlights) {
                const pageSpecificHighlight = Object.assign(Object.assign({}, highlight), { position: {
                        pageNumber,
                        boundingRect: highlight.position.boundingRect,
                        rects: [],
                        usePdfCoordinates: highlight.position.usePdfCoordinates,
                    } });
                let anyRectsOnPage = false;
                for (const rect of highlight.position.rects) {
                    if (pageNumber === (rect.pageNumber || highlight.position.pageNumber)) {
                        pageSpecificHighlight.position.rects.push(rect);
                        anyRectsOnPage = true;
                    }
                }
                if (anyRectsOnPage || pageNumber === highlight.position.pageNumber) {
                    groupedHighlights[pageNumber].push(pageSpecificHighlight);
                }
            }
        }
        return groupedHighlights;
    };
    const findOrCreateHighlightLayer = (page) => {
        const { textLayer } = (viewer === null || viewer === void 0 ? void 0 : viewer.getPageView(page - 1)) || {};
        if (!textLayer || textLayer === undefined)
            return null;
        return (0, pdfjs_dom_1.findOrCreateContainerLayer)(textLayer === null || textLayer === void 0 ? void 0 : textLayer.div, "PdfHighlighter__highlight-layer");
    };
    const scaledPositionToViewport = ({ pageNumber, boundingRect, rects, usePdfCoordinates, }) => {
        const viewport = viewer.getPageView(pageNumber - 1).viewport;
        return {
            boundingRect: (0, coordinates_1.scaledToViewport)(boundingRect, viewport, usePdfCoordinates),
            rects: (rects || []).map((rect) => (0, coordinates_1.scaledToViewport)(rect, viewport, usePdfCoordinates)),
            pageNumber,
        };
    };
    const renderHighlights = (nextProps) => {
        var _a;
        let hltTrnsf;
        let hlts = [];
        if (nextProps) {
            hltTrnsf = nextProps.highlightTransform;
            hlts = nextProps.highlights;
        }
        else {
            hltTrnsf = highlightTransform;
            hlts = highlights;
        }
        const highlightsByPage = groupHighlightsByPage(hlts);
        for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
            const highlightLayer = findOrCreateHighlightLayer(pageNumber);
            if (highlightLayer) {
                if (!roots.has(highlightLayer)) {
                    let root = (0, client_1.createRoot)(highlightLayer);
                    roots.set(highlightLayer, root);
                }
                (_a = roots.get(highlightLayer)) === null || _a === void 0 ? void 0 : _a.render(react_1.default.createElement("div", null, (highlightsByPage[pageNumber] || []).map((_a, index) => {
                    var { position, id } = _a, highlight = __rest(_a, ["position", "id"]);
                    // @ts-ignore
                    const viewportHighlight = Object.assign({ id, position: scaledPositionToViewport(position) }, highlight);
                    if (tip && tip.highlight.id === String(id)) {
                        showTip(tip.highlight, tip.callback(viewportHighlight));
                    }
                    const isScrolledTo = Boolean(scrolledToHighlightId === id);
                    return hltTrnsf(viewportHighlight, index, (highlight, callback) => {
                        setTip({ highlight, callback });
                        showTip(highlight, callback(highlight));
                    }, () => hideTip(), (rect) => {
                        const viewport = viewer.getPageView((rect.pageNumber || pageNumber) - 1).viewport;
                        return (0, coordinates_1.viewportToScaled)(rect, viewport);
                    }, (boundingRect) => {
                        const canvas = viewer.getPageView(pageNumber - 1).canvas;
                        return (0, get_area_as_png_1.default)(canvas, boundingRect);
                    }, isScrolledTo);
                })));
            }
        }
    };
    const onScroll = () => {
        onScrollChange();
        setScrolledToHighlightId("");
    };
    const scrollTo = (highlight) => {
        const { pageNumber, boundingRect, usePdfCoordinates } = highlight.position;
        viewer.container.removeEventListener("scroll", onScroll);
        const pageViewport = viewer.getPageView(pageNumber - 1).viewport;
        const scrollMargin = 10;
        viewer.scrollPageIntoView({
            pageNumber,
            destArray: [
                null,
                { name: "XYZ" },
                ...pageViewport.convertToPdfPoint(0, (0, coordinates_1.scaledToViewport)(boundingRect, pageViewport, usePdfCoordinates).top -
                    scrollMargin),
                0,
            ],
        });
        setScrolledToHighlightId(highlight.id);
        setTimeout(() => {
            viewer.container.addEventListener("scroll", onScroll);
        }, 100);
    };
    const hideTip = () => {
        setTipPosition(null);
        setTipChildren(null);
        setTip(null);
    };
    const showTip = (highlight, content) => {
        setTipPosition(highlight.position);
        setTipChildren(content);
    };
    // @ts-ignore
    const renderTip = () => {
        if (!tipPosition)
            return null;
        const { boundingRect, pageNumber } = tipPosition;
        if (viewerState) {
            const page = {
                node: viewerState.getPageView((boundingRect.pageNumber || pageNumber) - 1).div,
                pageNumber: boundingRect.pageNumber || pageNumber,
            };
            const pageBoundingClientRect = page.node.getBoundingClientRect();
            const pageBoundingRect = {
                bottom: pageBoundingClientRect.bottom,
                height: pageBoundingClientRect.height,
                left: pageBoundingClientRect.left,
                right: pageBoundingClientRect.right,
                top: pageBoundingClientRect.top,
                width: pageBoundingClientRect.width,
                x: pageBoundingClientRect.x,
                y: pageBoundingClientRect.y,
                pageNumber: page.pageNumber,
            };
            return (react_1.default.createElement(TipContainer_1.TipContainer, { scrollTop: viewerState.container.scrollTop, pageBoundingRect: pageBoundingRect, style: {
                    left: page.node.offsetLeft + boundingRect.left + boundingRect.width / 2,
                    top: boundingRect.top + page.node.offsetTop,
                    bottom: boundingRect.top + page.node.offsetTop + boundingRect.height,
                } }, tipChildren));
        }
    };
    return (react_1.default.createElement("div", null,
        react_1.default.createElement("div", { className: "PdfHighlighter", ref: documentRef, onContextMenu: (e) => e.preventDefault() },
            react_1.default.createElement("div", { className: "pdfViewer" }),
            renderTip())));
};
exports.PdfHighlighter = PdfHighlighter;
//# sourceMappingURL=PdfHighlighter.js.map