"use client";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useCallback, useEffect, useState } from "react";
import { EventBus, NullL10n, PDFLinkService, PDFViewer, } from "pdfjs-dist/legacy/web/pdf_viewer";
import "pdfjs-dist/web/pdf_viewer.css";
import "../style/pdf_viewer.css";
import "../style/PdfHighlighter.css";
import debounce from "lodash.debounce";
import { createRoot } from "react-dom/client";
import { scaledToViewport, viewportToScaled } from "../lib/coordinates";
import getAreaAsPng from "../lib/get-area-as-png";
import { findOrCreateContainerLayer } from "../lib/pdfjs-dom";
import { TipContainer } from "./TipContainer";
export const PdfHighlighter = ({ highlightTransform, highlights, onScrollChange, scrollRef, pdfDocument, pdfScaleValue = "auto", }) => {
    const [tip, setTip] = useState(null);
    const [tipPosition, setTipPosition] = useState(null);
    const [tipChildren, setTipChildren] = useState(null);
    const [scrolledToHighlightId, setScrolledToHighlightId] = useState("");
    const [containerNode, setContainerNode] = useState(null);
    const documentRef = React.useRef(null);
    const [viewerState, setViewerState] = useState();
    const [eventBus] = useState(new EventBus());
    var linkService;
    var viewer;
    var roots = new Map();
    const handleScaleValue = () => {
        if (viewer !== null && viewer !== undefined) {
            viewer.currentScaleValue = pdfScaleValue;
        }
    };
    const debouncedScaleValue = useCallback(debounce(handleScaleValue, 500), []);
    const resizeObserver = new ResizeObserver(debouncedScaleValue);
    useEffect(() => {
        var _a;
        const current = documentRef.current;
        setContainerNode(current);
        let doc;
        if (current) {
            let { ownerDocument: doc } = current;
            eventBus.on("textlayerrendered", () => {
                renderHighlights();
            });
            eventBus.on("pagesinit", () => {
                handleScaleValue();
                scrollRef(scrollTo);
            });
            (_a = doc === null || doc === void 0 ? void 0 : doc.defaultView) === null || _a === void 0 ? void 0 : _a.addEventListener("resize", debouncedScaleValue);
            if (resizeObserver)
                resizeObserver.observe(current);
            linkService = new PDFLinkService({
                eventBus: eventBus,
                externalLinkTarget: 2,
            });
            viewer = new PDFViewer({
                container: documentRef.current,
                eventBus: eventBus,
                textLayerMode: 2,
                removePageBorders: true,
                linkService: linkService,
                l10n: NullL10n,
            });
            setViewerState(viewer);
        }
        return () => {
            var _a;
            eventBus.off("textlayerrendered", renderHighlights);
            eventBus.off("pagesinit", () => {
                handleScaleValue();
                scrollRef(scrollTo);
            });
            (_a = doc === null || doc === void 0 ? void 0 : doc.defaultView) === null || _a === void 0 ? void 0 : _a.removeEventListener("resize", debouncedScaleValue);
        };
    }, []);
    useEffect(() => {
        if (viewer) {
            renderHighlights();
            viewer.setDocument(pdfDocument);
            linkService.setViewer(viewer);
            handleScaleValue();
        }
    }, [containerNode]);
    useEffect(() => {
        renderHighlights();
    }, [tip]);
    const groupHighlightsByPage = (highlights) => {
        const allHighlights = highlights.filter(Boolean);
        const pageNumbers = new Set();
        for (const highlight of allHighlights) {
            pageNumbers.add(highlight.position.pageNumber);
            for (const rect of highlight.position.rects) {
                if (rect.pageNumber) {
                    pageNumbers.add(rect.pageNumber);
                }
            }
        }
        const groupedHighlights = {};
        for (const pageNumber of pageNumbers) {
            groupedHighlights[pageNumber] = groupedHighlights[pageNumber] || [];
            for (const highlight of allHighlights) {
                const pageSpecificHighlight = Object.assign(Object.assign({}, highlight), { position: {
                        pageNumber,
                        boundingRect: highlight.position.boundingRect,
                        rects: [],
                        usePdfCoordinates: highlight.position.usePdfCoordinates,
                    } });
                let anyRectsOnPage = false;
                for (const rect of highlight.position.rects) {
                    if (pageNumber === (rect.pageNumber || highlight.position.pageNumber)) {
                        pageSpecificHighlight.position.rects.push(rect);
                        anyRectsOnPage = true;
                    }
                }
                if (anyRectsOnPage || pageNumber === highlight.position.pageNumber) {
                    groupedHighlights[pageNumber].push(pageSpecificHighlight);
                }
            }
        }
        return groupedHighlights;
    };
    const findOrCreateHighlightLayer = (page) => {
        const { textLayer } = (viewer === null || viewer === void 0 ? void 0 : viewer.getPageView(page - 1)) || {};
        if (!textLayer || textLayer === undefined)
            return null;
        return findOrCreateContainerLayer(textLayer === null || textLayer === void 0 ? void 0 : textLayer.div, "PdfHighlighter__highlight-layer");
    };
    const scaledPositionToViewport = ({ pageNumber, boundingRect, rects, usePdfCoordinates, }) => {
        const viewport = viewer.getPageView(pageNumber - 1).viewport;
        return {
            boundingRect: scaledToViewport(boundingRect, viewport, usePdfCoordinates),
            rects: (rects || []).map((rect) => scaledToViewport(rect, viewport, usePdfCoordinates)),
            pageNumber,
        };
    };
    const renderHighlights = (nextProps) => {
        var _a;
        let hltTrnsf;
        let hlts = [];
        if (nextProps) {
            hltTrnsf = nextProps.highlightTransform;
            hlts = nextProps.highlights;
        }
        else {
            hltTrnsf = highlightTransform;
            hlts = highlights;
        }
        const highlightsByPage = groupHighlightsByPage(hlts);
        for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
            const highlightLayer = findOrCreateHighlightLayer(pageNumber);
            if (highlightLayer) {
                if (!roots.has(highlightLayer)) {
                    let root = createRoot(highlightLayer);
                    roots.set(highlightLayer, root);
                }
                (_a = roots.get(highlightLayer)) === null || _a === void 0 ? void 0 : _a.render(React.createElement("div", null, (highlightsByPage[pageNumber] || []).map((_a, index) => {
                    var { position, id } = _a, highlight = __rest(_a, ["position", "id"]);
                    // @ts-ignore
                    const viewportHighlight = Object.assign({ id, position: scaledPositionToViewport(position) }, highlight);
                    if (tip && tip.highlight.id === String(id)) {
                        showTip(tip.highlight, tip.callback(viewportHighlight));
                    }
                    const isScrolledTo = Boolean(scrolledToHighlightId === id);
                    return hltTrnsf(viewportHighlight, index, (highlight, callback) => {
                        setTip({ highlight, callback });
                        showTip(highlight, callback(highlight));
                    }, () => hideTip(), (rect) => {
                        const viewport = viewer.getPageView((rect.pageNumber || pageNumber) - 1).viewport;
                        return viewportToScaled(rect, viewport);
                    }, (boundingRect) => {
                        const canvas = viewer.getPageView(pageNumber - 1).canvas;
                        return getAreaAsPng(canvas, boundingRect);
                    }, isScrolledTo);
                })));
            }
        }
    };
    const onScroll = () => {
        onScrollChange();
        setScrolledToHighlightId("");
    };
    const scrollTo = (highlight) => {
        const { pageNumber, boundingRect, usePdfCoordinates } = highlight.position;
        viewer.container.removeEventListener("scroll", onScroll);
        const pageViewport = viewer.getPageView(pageNumber - 1).viewport;
        const scrollMargin = 10;
        viewer.scrollPageIntoView({
            pageNumber,
            destArray: [
                null,
                { name: "XYZ" },
                ...pageViewport.convertToPdfPoint(0, scaledToViewport(boundingRect, pageViewport, usePdfCoordinates).top -
                    scrollMargin),
                0,
            ],
        });
        setScrolledToHighlightId(highlight.id);
        setTimeout(() => {
            viewer.container.addEventListener("scroll", onScroll);
        }, 100);
    };
    const hideTip = () => {
        setTipPosition(null);
        setTipChildren(null);
        setTip(null);
    };
    const showTip = (highlight, content) => {
        setTipPosition(highlight.position);
        setTipChildren(content);
    };
    // @ts-ignore
    const renderTip = () => {
        if (!tipPosition)
            return null;
        const { boundingRect, pageNumber } = tipPosition;
        if (viewerState) {
            const page = {
                node: viewerState.getPageView((boundingRect.pageNumber || pageNumber) - 1).div,
                pageNumber: boundingRect.pageNumber || pageNumber,
            };
            const pageBoundingClientRect = page.node.getBoundingClientRect();
            const pageBoundingRect = {
                bottom: pageBoundingClientRect.bottom,
                height: pageBoundingClientRect.height,
                left: pageBoundingClientRect.left,
                right: pageBoundingClientRect.right,
                top: pageBoundingClientRect.top,
                width: pageBoundingClientRect.width,
                x: pageBoundingClientRect.x,
                y: pageBoundingClientRect.y,
                pageNumber: page.pageNumber,
            };
            return (React.createElement(TipContainer, { scrollTop: viewerState.container.scrollTop, pageBoundingRect: pageBoundingRect, style: {
                    left: page.node.offsetLeft + boundingRect.left + boundingRect.width / 2,
                    top: boundingRect.top + page.node.offsetTop,
                    bottom: boundingRect.top + page.node.offsetTop + boundingRect.height,
                } }, tipChildren));
        }
    };
    return (React.createElement("div", null,
        React.createElement("div", { className: "PdfHighlighter", ref: documentRef, onContextMenu: (e) => e.preventDefault() },
            React.createElement("div", { className: "pdfViewer" }),
            renderTip())));
};
//# sourceMappingURL=PdfHighlighter.js.map